<?php

namespace jpuck\avhost\Core\Utils;

use jpuck\avhost\Core\Contracts\Exportable;
use jpuck\avhost\Core\Traits\EncodeFromArray;
use jpuck\avhost\Core\Traits\Configurable;

class Signature implements Exportable
{
    use EncodeFromArray;
    use Configurable;

    protected $version;
    protected $createdAt;
    protected $createdBy;

    protected $header = <<<HEADER
##########################################
# Generated by avhost
# https://github.com/jpuck/avhost

HEADER;

    protected $footer = <<<FOOTER
##########################################

FOOTER;

    public function __construct()
    {
        $this->setDefaultValues();
    }

    public function getVersion() : string
    {
        return $this->version;
    }

    public function setVersion(string $version) : Signature
    {
        $this->version = $version;

        return $this;
    }

    public function getCreatedAt() : string
    {
        return $this->createdAt;
    }

    public function setCreatedAt(string $createdAt) : Signature
    {
        $this->createdAt = $createdAt;

        return $this;
    }

    public function getCreatedBy() : string
    {
        return $this->createdBy;
    }

    public function setCreatedBy(string $createdBy) : Signature
    {
        $this->createdBy = $createdBy;

        return $this;
    }

    public function toArray() : array
    {
        return [
            'version' => $this->getVersion(),
            'createdAt' => $this->getCreatedAt(),
            'createdBy' => $this->getCreatedBy(),
        ];
    }

    public function render() : string
    {
        $string = '';

        foreach ($this->toArray() as $name => $value) {
            $string .= $this->getKeyValueLine($name, $value);
        }

        if ($configuration = $this->getConfiguration()) {
            $string .= $this->getKeyValueLine('contentHash', $this->getContentHash());
            $string .= $this->getKeyValueLine('configuration', $configuration->toBase64());
        }

        return $this->header . $string . $this->footer;
    }

    protected function setDefaultValues()
    {
        $this->setVersion((new Version)->getVersion());

        $this->setCreatedAt(date('c'));

        $this->setCreatedBy(trim(`whoami`).'@'.gethostname());
    }

    protected function getContentHash() : string
    {
        return sha1($this->getConfiguration()->getApplicator()->render());
    }

    protected function getKeyValueLine(string $key, string $value) : string
    {
        return "# $key $value".PHP_EOL;
    }
}
